#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File: loop.py.py
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""
import platform
import re
from subprocess import Popen, PIPE
import sys
import pinyin
import uuid
from client.client_log.logging import logging


def to_native_str(text):
    if sys.version_info[0] >= 3:
        if isinstance(text, bytes):
            try:
                text = text.decode('utf-8')
            except ValueError:
                try:
                    text = text.decode('gb2312')
                    text = pinyin.get(text, format='strip')
                except ValueError:
                    text = uuid.getnode()

    else:
        if isinstance(text, unicode):  # noqa: F821
            text = text.encode('utf-8')

    return text


def os_and_user_name():
    name = ''
    if platform.system() == 'Linux':
        hostname = _exec_cmd('hostname')
        user = _exec_cmd('whoami')
        name = re.sub(r'[\n\r\\ ]', '', f'{hostname}@{user}')
    elif platform.system() == 'Windows':
        name = re.sub(r'[\n\r\\ ]', '', _exec_cmd('whoami'))

    logging.info(f'get os_and_user name of this machine [{name}]')
    return name


def _exec_cmd(cmd):
    if isinstance(cmd, str):
        cmd = [cmd]

    p = Popen(cmd, stdout=PIPE, stderr=PIPE)
    ret, err = p.communicate()
    ret = to_native_str(ret)
    return ret



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File: WxContext.py.py
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""
import os
import wx
import wx.adv
import platform


class NormalUi(wx.Frame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UI(wx.Frame):
    def __init__(self, *args, **kwargs):
        kwargs['style'] = wx.MINIMIZE_BOX
        super().__init__(*args, **kwargs)

    def create_menu_bar(self, bind_func):
        menu_bar = wx.MenuBar()
        register_menu = wx.Menu()
        register_item = register_menu.Append(wx.ID_NEW, 'Register',
                                             helpString='register to add new '
                                                        'user')
        menu_bar.Append(register_menu, '&Register')
        self.SetMenuBar(menu_bar)

        self.Bind(wx.EVT_MENU, bind_func, register_item)
        return menu_bar

    def create_question_dialog(self, message='Demo message', exist=True):
        dlg = wx.MessageDialog(
            None, message=message,
            caption='Message',
            style=wx.YES_NO | wx.ICON_QUESTION)
        result = dlg.ShowModal()
        if result == wx.ID_NO and exist:
            if TaskBar.is_system_tray_available():
                self.Show(False)
            else:
                self.Close()

    def create_ok_dialog(self, message='Demo message'):
        dlg = wx.MessageDialog(None, message=message,
                               caption='Message',
                               style=wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()


class LogUI(UI):
    def __init__(self, exe_path, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._exe_path = exe_path
        self.panelTitleBar = wx.Panel(self, wx.ID_ANY)
        self.btnMinimize = wx.Button(self.panelTitleBar, wx.ID_ANY, "",
                                     style=wx.BORDER_NONE | wx.BU_NOTEXT)
        self.btnMaximize = wx.Button(self.panelTitleBar, wx.ID_ANY, "",
                                     style=wx.BORDER_NONE | wx.BU_NOTEXT)

        self.btnExit = wx.Button(self.panelTitleBar, wx.ID_ANY, "",
                                 style=wx.BORDER_NONE | wx.BU_NOTEXT)
        self.panelBody = wx.Panel(self, wx.ID_ANY)

        self.Bind(wx.EVT_BUTTON, self.OnBtnExitClick, self.btnExit)
        self.Bind(wx.EVT_BUTTON, self.OnBtnMinimizeClick, self.btnMinimize)
        self.Bind(wx.EVT_BUTTON, self.OnBtnMaximizeClick, self.btnMaximize)
        self.panelTitleBar.Bind(wx.EVT_LEFT_DOWN, self.OnTitleBarLeftDown)
        self.panelTitleBar.Bind(wx.EVT_MOTION, self.OnMouseMove)

        self.server = wx.StaticText(self.panelBody, wx.ID_ANY,
                                    label="Server URL:")
        self.user = wx.StaticText(self.panelBody, wx.ID_ANY,
                                  label="User Name:")
        self.password = wx.StaticText(self.panelBody, wx.ID_ANY,
                                      label="Password:")

        self.btn_sign_in = wx.Button(self.panelBody, wx.ID_ANY, "Sign In",
                                     size=(80, 38))
        self.btn_sign_up = wx.Button(self.panelBody, wx.ID_ANY, "Sign Up",
                                     size=(80, 38))
        self.btn_sign_out = wx.Button(self.panelBody, wx.ID_ANY, "Sign Out",
                                      size=(80, 38))

        self.server_text = wx.TextCtrl(self.panelBody, wx.ID_ANY,
                                       size=wx.Size(230, 40))
        self.server_text.SetBackgroundColour(wx.Colour(230, 230, 250))
        _font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.server.SetFont(_font)

        self.user_text = wx.TextCtrl(self.panelBody, wx.ID_ANY,
                                     size=wx.Size(230, 40))
        self.user_text.SetBackgroundColour(wx.Colour(230, 230, 250))
        _font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.user.SetFont(_font)

        self.password_text = wx.TextCtrl(self.panelBody, wx.ID_ANY,
                                         size=wx.Size(230, 40),
                                         style=wx.TE_PASSWORD)
        self.password_text.SetBackgroundColour(wx.Colour(230, 230, 250))
        _font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.password.SetFont(_font)

        self.save_password = wx.CheckBox(
            self.panelBody, wx.ID_ANY, label="Save Password")
        self.btn_forget_password = wx.Button(
            self.panelBody, style=wx.BORDER_NONE, label="forget password?")
        self.btn_forget_password.SetBackgroundColour(wx.Colour(255, 255, 255))
        self.skybility_logo = wx.StaticBitmap(
            self.panelBody, wx.ID_ANY,
            wx.Bitmap(
                os.path.join(self._exe_path, "img", "skybility_logo.png"),
                wx.BITMAP_TYPE_ANY))

        self._isClickedDown = False
        self._LastPosition = self.GetPosition()

        self.__set_properties()
        self.__do_layout()

    def __set_properties(self):
        # self.SetTitle("frame")
        size = (22, 22)
        if platform.system() == 'Linux':
            size = (32, 22)

        self.btnMinimize.SetMinSize(size)
        self.btnMinimize.SetBitmap(
            wx.Bitmap(os.path.join(self._exe_path, "img", "min.png"),
                      wx.BITMAP_TYPE_ANY))
        self.btnMaximize.SetMinSize(size)
        self.btnMaximize.SetBitmap(
            wx.Bitmap(os.path.join(self._exe_path, "img", "max.png"),
                      wx.BITMAP_TYPE_ANY))
        self.btnExit.SetMinSize(size)
        self.btnExit.SetBitmap(
            wx.Bitmap(os.path.join(self._exe_path, "img", "close.png"),
                      wx.BITMAP_TYPE_ANY))
        self.panelTitleBar.SetBackgroundColour(wx.Colour(44, 134, 179))
        self.panelBody.SetBackgroundColour(wx.Colour(255, 255, 255))

    def __do_layout(self):

        # Sizers:
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        grid_sizer_1 = wx.FlexGridSizer(2, 1, 0, 0)
        sizerTitleBar = wx.FlexGridSizer(1, 4, 0, 0)

        body_sizer = wx.BoxSizer(wx.VERTICAL)
        body_sizer1 = wx.FlexGridSizer(4, 3, 30, 0)
        body_sizer2 = wx.BoxSizer(wx.HORIZONTAL)

        # Titlebar:
        bitmap = wx.Bitmap(os.path.join(
            self._exe_path, "img", "login_universal_copy_paste_tools.png"))

        iconTitleBar = wx.StaticBitmap(self.panelTitleBar, wx.ID_ANY,
                                       bitmap)

        sizerTitleBar.Add(iconTitleBar, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)

        sizerTitleBar.Add(self.btnMinimize, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 2)
        sizerTitleBar.Add(self.btnMaximize, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 2)
        sizerTitleBar.Add(self.btnExit, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 2)
        sizerTitleBar.AddGrowableRow(0)
        sizerTitleBar.AddGrowableCol(0)

        body_sizer1.Add(
            self.server, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 2)
        body_sizer1.Add(
            self.server_text, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        body_sizer1.Add(
            self.btn_sign_in, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        body_sizer1.Add(
            self.user, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 2)
        body_sizer1.Add(
            self.user_text, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        body_sizer1.Add(
            self.btn_sign_up, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        body_sizer1.Add(
            self.password, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 2)
        body_sizer1.Add(
            self.password_text, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        body_sizer1.Add(
            self.btn_sign_out, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        body_sizer1.Add(
            self.save_password, 1, wx.ALIGN_CENTER | wx.CENTER, 5)
        body_sizer1.Add(
            self.btn_forget_password, 1, wx.ALIGN_CENTER | wx.CENTER, 5)
        body_sizer1.Add(
            self.skybility_logo, 1, wx.ALIGN_CENTER | wx.CENTER, 5)

        body_sizer1.AddGrowableCol(2)
        body_sizer1.AddGrowableRow(3)
        body_sizer2.Add(body_sizer1,
                        flag=wx.ALIGN_CENTER_VERTICAL | wx.RIGHT | wx.LEFT)
        body_sizer.AddStretchSpacer(prop=1)
        body_sizer.Add(body_sizer2, 0, wx.CENTER)
        body_sizer.AddStretchSpacer(prop=1)

        self.panelTitleBar.SetSizer(sizerTitleBar)
        self.panelBody.SetSizer(body_sizer)

        grid_sizer_1.Add(self.panelTitleBar, 1, wx.EXPAND, 0)
        grid_sizer_1.Add(self.panelBody, 1, wx.EXPAND, 0)
        grid_sizer_1.AddGrowableRow(1)
        grid_sizer_1.AddGrowableCol(0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_1)

    def OnTitleBarLeftDown(self, event):
        self._LastPosition = event.GetPosition()

    def OnBtnExitClick(self, event):
        # self.Close()
        if not TaskBar.is_system_tray_available():
            wx.Exit()

        self.Hide()
        return False

    def OnBtnMinimizeClick(self, event):
        self.Iconize(True)

    def OnBtnMaximizeClick(self, event):
        self.Maximize(not self.IsMaximized())

    def OnMouseMove(self, event):
        if event.Dragging():
            mouse_x, mouse_y = wx.GetMousePosition()
            self.Move(mouse_x - self._LastPosition[0],
                      mouse_y - self._LastPosition[1])


class RegistUI(UI):
    def __init__(self, exe_path, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._exe_path = exe_path
        self.panelTitleBar = wx.Panel(self, wx.ID_ANY)
        self.btnMinimize = wx.Button(self.panelTitleBar, wx.ID_ANY, "",
                                     style=wx.BORDER_NONE | wx.BU_NOTEXT)
        self.btnMaximize = wx.Button(self.panelTitleBar, wx.ID_ANY, "",
                                     style=wx.BORDER_NONE | wx.BU_NOTEXT)

        self.btnExit = wx.Button(self.panelTitleBar, wx.ID_ANY, "",
                                 style=wx.BORDER_NONE | wx.BU_NOTEXT)
        self.panelBody = wx.Panel(self, wx.ID_ANY)

        self.Bind(wx.EVT_BUTTON, self.OnBtnExitClick, self.btnExit)
        self.Bind(wx.EVT_BUTTON, self.OnBtnMinimizeClick, self.btnMinimize)
        self.Bind(wx.EVT_BUTTON, self.OnBtnMaximizeClick, self.btnMaximize)
        self.panelTitleBar.Bind(wx.EVT_LEFT_DOWN, self.OnTitleBarLeftDown)
        self.panelTitleBar.Bind(wx.EVT_MOTION, self.OnMouseMove)

        self.user_name = wx.StaticText(self.panelBody, wx.ID_ANY,
                                       label="username: ")
        self.password = wx.StaticText(self.panelBody, wx.ID_ANY,
                                      label="password: ")
        self.repeat_pw = wx.StaticText(self.panelBody, wx.ID_ANY,
                                       label="repeatpw: ")

        self.username_text = wx.TextCtrl(self.panelBody, wx.ID_ANY,
                                         size=wx.Size(230, 40))
        self.username_text.SetBackgroundColour(wx.Colour(230, 230, 250))
        _font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.username_text.SetFont(_font)

        self.pw_text = wx.TextCtrl(self.panelBody, wx.ID_ANY,
                                   size=wx.Size(230, 40),
                                   style=wx.TE_PASSWORD)
        self.pw_text.SetBackgroundColour(wx.Colour(230, 230, 250))
        _font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.pw_text.SetFont(_font)

        self.repw_text = wx.TextCtrl(self.panelBody, wx.ID_ANY,
                                     size=wx.Size(230, 40),
                                     style=wx.TE_PASSWORD)
        self.repw_text.SetBackgroundColour(wx.Colour(230, 230, 250))
        _font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.repw_text.SetFont(_font)

        self.btn_ok = wx.Button(self.panelBody, wx.ID_ANY, label="OK",
                                size=(80, 38))
        self.btn_ok.SetBackgroundColour(wx.Colour(44, 134, 179))
        self.btn_ok.SetForegroundColour(wx.Colour(255, 255, 255))

        self._isClickedDown = False
        self._LastPosition = self.GetPosition()

        self.__set_properties()
        self.__do_layout()

        self.Centre()
        self.Show(True)

    def __set_properties(self):
        self.SetTitle("frame")
        size = (22, 22)
        if platform.system() == 'Linux':
            size = (32, 22)

        self.btnMinimize.SetMinSize(size)
        self.btnMinimize.SetBitmap(
            wx.Bitmap(os.path.join(self._exe_path, "img", "min.png"),
                      wx.BITMAP_TYPE_ANY))
        self.btnMaximize.SetMinSize(size)
        self.btnMaximize.SetBitmap(
            wx.Bitmap(os.path.join(self._exe_path, "img", "max.png"),
                      wx.BITMAP_TYPE_ANY))
        self.btnExit.SetMinSize(size)
        self.btnExit.SetBitmap(
            wx.Bitmap(os.path.join(self._exe_path, "img", "close.png"),
                      wx.BITMAP_TYPE_ANY))
        self.panelTitleBar.SetBackgroundColour(wx.Colour(44, 134, 179))
        self.panelBody.SetBackgroundColour(wx.Colour(255, 255, 255))

    def __do_layout(self):

        # Sizers:
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        grid_sizer_1 = wx.FlexGridSizer(2, 1, 0, 0)
        sizerTitleBar = wx.FlexGridSizer(1, 4, 0, 0)

        body_sizer = wx.BoxSizer(wx.VERTICAL)
        body_sizer1 = wx.FlexGridSizer(4, 2, 30, 15)
        body_sizer2 = wx.BoxSizer(wx.HORIZONTAL)

        # Titlebar:
        bitmap = wx.Bitmap(os.path.join(self._exe_path,
                                        "img", "reg_registeruser.png"))

        iconTitleBar = wx.StaticBitmap(self.panelTitleBar, wx.ID_ANY,
                                       bitmap)

        sizerTitleBar.Add(iconTitleBar, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)

        sizerTitleBar.Add(self.btnMinimize, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 2)
        sizerTitleBar.Add(self.btnMaximize, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 2)
        sizerTitleBar.Add(self.btnExit, 0,
                          wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 2)
        sizerTitleBar.AddGrowableRow(0)
        sizerTitleBar.AddGrowableCol(0)

        body_sizer1.Add(
            self.user_name, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 2)
        body_sizer1.Add(
            self.username_text, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        body_sizer1.Add(
            self.password, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        body_sizer1.Add(
            self.pw_text, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL,
            2)
        body_sizer1.Add(
            self.repeat_pw, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        body_sizer1.Add(
            self.repw_text, 1,
            wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        body_sizer1.AddStretchSpacer(prop=1)
        body_sizer1.Add(
            self.btn_ok, 1, wx.ALIGN_CENTER | wx.CENTER, 5)
        # body_sizer1.Add(
        #     self.skybility_logo, 1, wx.ALIGN_CENTER | wx.CENTER, 5)

        body_sizer1.AddGrowableCol(1)
        body_sizer1.AddGrowableRow(3)
        body_sizer2.Add(body_sizer1,
                        flag=wx.ALIGN_CENTER_VERTICAL | wx.RIGHT | wx.LEFT)
        body_sizer.AddStretchSpacer(prop=1)
        body_sizer.Add(body_sizer2, 0, wx.CENTER)
        body_sizer.AddStretchSpacer(prop=1)

        self.panelTitleBar.SetSizer(sizerTitleBar)
        self.panelBody.SetSizer(body_sizer)

        grid_sizer_1.Add(self.panelTitleBar, 1, wx.EXPAND, 0)
        grid_sizer_1.Add(self.panelBody, 1, wx.EXPAND, 0)
        grid_sizer_1.AddGrowableRow(1)
        grid_sizer_1.AddGrowableCol(0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_1)

    def OnTitleBarLeftDown(self, event):
        self._LastPosition = event.GetPosition()

    def OnBtnExitClick(self, event):
        self.Close()

    def OnBtnMinimizeClick(self, event):
        self.Iconize(True)

    def OnBtnMaximizeClick(self, event):
        self.Maximize(not self.IsMaximized())

    def OnMouseMove(self, event):
        if event.Dragging():
            mouse_x, mouse_y = wx.GetMousePosition()
            self.Move(mouse_x - self._LastPosition[0],
                      mouse_y - self._LastPosition[1])


class TaskBar:
    def __init__(self, system_tray):
        self.system_tray = system_tray

    @classmethod
    def is_system_tray_available(cls):
        bar = wx.adv.TaskBarIcon()
        return bar.IsAvailable()

    def create_menu_item(self, menu: wx.Menu, label, func):
        menu.AppendSeparator()
        item = wx.MenuItem(menu, -1, label)
        menu.Bind(wx.EVT_MENU, func, id=item.GetId())
        menu.Append(item)

        if label == "Start Service":
            item.Enable(self.system_tray.app.service_started is False)
        if label == "Stop Service":
            item.Enable(self.system_tray.app.service_started is True)

        return item

    def chk_item_click(self, menu, label, func, item_list=[]):
        menu.AppendSeparator()
        _menu = wx.Menu()
        if label == "Recent Content":
            for x in range(len(item_list)):
                client_1 = _menu.AppendCheckItem(wx.NewId(),
                                                 f'{x}: '
                                                 f'{item_list[x][21:31]}...')
                _menu.Bind(wx.EVT_MENU, func(x), id=client_1.GetId())

        elif label == "Client List":
            for x in item_list.current_clients:
                client_2 = _menu.AppendCheckItem(wx.NewId(), x)
                client_2.Check(x not in item_list.ignore_clients)
                _menu.Bind(wx.EVT_MENU, func(x), id=client_2.GetId())

        menu.Append(wx.ID_ANY, label, _menu)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File:
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""
import os
import threading
import base64
import json
import wx
import wx.adv
import re
import platform
from pubsub import pub
from client.pywx.WxContext import TaskBar, LogUI, RegistUI, NormalUi
from client.uri.uri import AppRequest, SocketClient
from client.client_log.logging import logging
from client import exe_path, user_home

logging.init_log(log_level=logging.WARNING, logger_name='engineio.client',
                 log_dir=os.path.join(user_home, 'ucnp_client', 'log'))
logging.init_log(log_level=logging.WARN, logger_name='socketio.client',
                 log_dir=os.path.join(user_home, 'ucnp_client', 'log'))
logging.init_log(log_level=logging.INFO,
                 log_dir=os.path.join(user_home, 'ucnp_client', 'log'))


class AppSend:
    def __init__(self, url):
        self._url: str = url
        self._valid_url()
        self._app = AppRequest(self._url)

    @property
    def app(self):
        return self._app

    def _valid_url(self):
        if not (self._url.startswith('http') | self._url.startswith('https')):
            self._url = 'http://' + self._url


class SockSend:
    def __init__(self, url, pipe_data, connection):
        self._url = url
        self._valid_url()
        self._sock = SocketClient(self._url, pipe_data, connection)

    @property
    def sock(self):
        return self._sock

    def _valid_url(self):
        if not (self._url.startswith('http') | self._url.startswith('https')):
            self._url = 'http://' + self._url


class App(wx.App):
    def OnExit(self):
        # LoginUI.graceful_exit()
        return 0


class LoginGui(LogUI):
    def __init__(self, pipe_data, connection, *args, **kwargs):
        self._server_url = ''
        self._username = ''
        self._pw = ''
        # self.window = None

        self._exe_path = exe_path
        self.pipe_data = pipe_data()
        self.connection = connection()
        self.service_started = False

        self._sender_thread = None
        self._receiver_thread = None
        self._socker = None

        conf_dir = os.path.join(user_home, 'ucnp_conf')
        if not os.path.exists(conf_dir):
            os.makedirs(conf_dir, exist_ok=True)

        self._connect_string_file = os.path.join(conf_dir,
                                                 'connect_string.txt')
        self._enable_save_pw_file = os.path.join(conf_dir,
                                                 'enable_save_password.txt')

        # kwargs["style"] = kwargs.get("style", 0)
        super().__init__(self._exe_path, *args, **kwargs)

        pub.subscribe(self.response_to_request_url, 'request_server_url')

        self.bind_btn()
        self.init_value()
        self.disable_sign_button()

        # problem unknown in windows
        if platform.system() == 'Windows':
            self.Iconize(True)
            self.Iconize(False)

    @property
    def exe_path(self):
        return self._exe_path

    def init_value(self):
        if not self.is_save_pw_enabled():
            return False

        if not os.path.exists(self._connect_string_file):
            return False

        ret = ['', '', '']
        with open(self._connect_string_file, 'r') as f:
            try:
                ret = json.load(f)
            except (TypeError, json.JSONDecodeError):
                pass

        server_url = base64.b64decode(ret[0].encode('utf-8'))
        user = base64.b64decode(ret[1].encode('utf-8'))
        word = base64.b64decode(ret[2].encode('utf-8'))

        self.server_text.SetValue(server_url)
        self.user_text.SetValue(user)
        self.password_text.SetValue(word)
        self.save_password.SetValue(True)
        return True

    def response_to_request_url(self, message):
        pub.sendMessage('server_url', message=self._server_url)

    def bind_btn(self):
        # current forget password is not available
        self.btn_forget_password.Disable()

        self.server_text.Bind(wx.EVT_TEXT, self.server_url)
        self.user_text.Bind(wx.EVT_TEXT, self.set_username)
        self.password_text.Bind(wx.EVT_TEXT, self.set_password)

        self.btn_sign_in.Bind(wx.EVT_BUTTON, self.login)
        self.btn_sign_up.Bind(wx.EVT_BUTTON, self.register)
        self.btn_sign_out.Bind(wx.EVT_BUTTON, self.logout)

        self.btn_forget_password.Bind(wx.EVT_BUTTON, self.find_password)
        self.save_password.Bind(wx.EVT_CHECKBOX, self.save_pw)

        self.Show(True)
        self.Center()

    def cook_tray(self, window):
        if not TaskBar.is_system_tray_available():
            return

        frame = wx.Frame(self)
        TaskBarIcon(self, frame)

    def server_url(self, event):
        obj = event.GetEventObject()
        self._server_url = obj.GetValue()

    def validate(self):
        # valid host ip
        if not self._server_url:
            self.create_ok_dialog('host or port should not be empty')
            logging.error('host or port should not be empty')
            return False

        # valid port
        return True

    def set_username(self, event):
        obj = event.GetEventObject()
        self._username = obj.GetValue()

    def set_password(self, event):
        obj = event.GetEventObject()
        self._pw = obj.GetValue()

    def try_login_without_message(self):
        token = AppSend(self._server_url).app.login(self._username,
                                                    self._pw)
        if token.get('code') == 0:
            arg = 'Bearer ' + token.get('data', {}).get('access_token', '')

            self._socker = SockSend(
                self._server_url, self.pipe_data, self.connection).sock
            self._socker.connect(arg)

            self._sender_thread = threading.Thread(
                target=self._socker.start, daemon=True)
            self._sender_thread.start()

            self._receiver_thread = threading.Thread(
                target=self._socker.register_namespace, daemon=True)
            self._receiver_thread.start()

            message = f'Succeed login into server [{self._server_url}]'
            logging.info(message)
            self._save_pw_to_file()
            self.service_started = True

            self._hide_gui()

        return token

    def login(self, event):
        logging.info(f'Login to server [{self._server_url}]')
        if not self.validate():
            self.disable_sign_button()
            return

        if self.service_started:
            self.disable_sign_button()
            self._hide_gui()
            logging.info(
                'Service has already started, ignore start action this time')
            return

        token = self.try_login_without_message()
        if token.get('code') == 0:
            self.disable_sign_button()
            return

        else:
            message = token.get('message', '')
            logging.info(
                f'Failed login into server [{self._server_url}]')
            self.create_ok_dialog(message + ",check your input.")
            # self.Close(True)

    def register(self, event):
        if not self._server_url:
            self.create_ok_dialog('Server URL not set')
            return

        RegisterUI(self.exe_path, self, wx.ID_ANY, "", size=(559, 450))

    def system_tray(self, event):
        self.Hide()

    def _hide_gui(self):
        if TaskBar.is_system_tray_available():
            self.Show(False)
        else:
            self.Iconize(True)

    def find_password(self, event):
        pass

    def save_pw(self, event):
        obj = event.GetEventObject()
        choose = obj.GetValue()
        if choose:
            self.enable_save_pw(True)
        else:
            self.enable_save_pw(False)

    def _save_pw_to_file(self):
        if self.save_password.GetValue():
            file = self._connect_string_file
            url = self._server_url.encode('utf-8')
            user = self._username.encode('utf-8')
            word = self._pw.encode('utf-8')
            data = (base64.b64encode(url).decode('utf-8'),
                    base64.b64encode(user).decode('utf-8'),
                    base64.b64encode(word).decode('utf-8')
                    )
            with open(file, 'w') as f:
                json.dump(data, f)

    def enable_save_pw(self, ret):
        file = self._enable_save_pw_file
        if not ret:
            with open(self._connect_string_file, 'w') as f:
                f.truncate()

        ret = str(ret)
        with open(file, 'w') as f:
            f.write(ret)

    def is_save_pw_enabled(self):
        file = self._enable_save_pw_file
        if not os.path.exists(file):
            return False

        with open(file, 'r') as f:
            if f.read() != 'True':
                return False

        return True

    def logout(self, event):
        if self._socker:
            self._socker.stop()

        self.service_started = False
        self.disable_sign_button()

    def stop(self):
        if self._socker:
            self._socker.stop()

        self.service_started = False

    def OnBtnExitClick(self, event):
        if not TaskBar.is_system_tray_available():
            self.stop()
            wx.Exit()

        self.Hide()
        return False

    def disable_sign_button(self):
        sign_in_label = {True: "Signed In", False: "Sign In"}
        sign_out_label = {True: "Sign Out", False: "Signed Out"}
        self.btn_sign_in.Enable(not self.service_started)
        self.btn_sign_out.Enable(self.service_started)

        self.btn_sign_out.SetLabel(sign_out_label.get(self.service_started))
        self.btn_sign_in.SetLabel(sign_in_label.get(self.service_started))


class RegisterUI(RegistUI):
    def __init__(self, exe_path, *args, **kwargs):
        # kwargs['style'] = 0
        super().__init__(exe_path, *args, **kwargs)

        self.user_name = ''
        self.pass_word = ''
        self.confirm_password = ''
        self.server_url = ''

        pub.subscribe(self.end_refresh_url, 'server_url')

        self.request_refresh_url()
        self.bind_btn()

        # problem unknown in windows
        if platform.system() == 'Windows':
            self.Iconize(True)
            self.Iconize(False)

    def request_refresh_url(self):
        pub.sendMessage('request_server_url', message='register ui')

    def end_refresh_url(self, message):
        self.server_url = message
        if not self.server_url:
            self.create_ok_dialog(
                'Server URL not set, go back to the login ui and set it!')

    def bind_btn(self):
        self.btn_ok.Bind(wx.EVT_BUTTON, self.confirm)
        self.username_text.Bind(wx.EVT_TEXT, self.username)
        self.pw_text.Bind(wx.EVT_TEXT, self.set_password)
        self.repw_text.Bind(wx.EVT_TEXT, self.confirm_passwords)

    def username(self, event):
        obj = event.GetEventObject()
        self.user_name = obj.GetValue()

    def set_password(self, event):
        obj = event.GetEventObject()
        self.pass_word = obj.GetValue()

    def confirm_passwords(self, event):
        obj = event.GetEventObject()
        self.confirm_password = obj.GetValue()

    def _valid_pw(self):
        _c = re.compile(r'^(?:(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])).{8,40}$')
        ret = _c.match(self.pass_word)
        return ret

    def confirm(self, event):
        if not self._valid_pw():
            message = 'Password not satisfy rules, \n' \
                      'must include [A-Za-z0-9] and length to 8 char,\n' \
                      'input again !!'
            self.create_question_dialog(message)
            return

        if self.confirm_password and self.pass_word:
            if self.confirm_password == self.pass_word:
                AppSend(self.server_url).app.register(
                    self.user_name,
                    self.pass_word,
                    self.confirm_password)
                self.Close()
                return

        message = 'password and confirm password mismatch,\nyou need input ' \
                  'again'
        self.create_question_dialog(message)


class TaskBarIcon(wx.adv.TaskBarIcon):
    # TRAY_HOST_INFO = ''

    def __init__(self, app: LoginGui, frame):
        self._frame = frame
        super().__init__()
        self._app = app
        self._tray_icon = os.path.join(
            self._app.exe_path, 'img', 'task_bar.png')
        # TaskBarIcon.TRAY_HOST_INFO = '{0}'.format('99')
        self.set_icon(self._tray_icon)
        self.Bind(wx.adv.EVT_TASKBAR_LEFT_DOWN, self.show_balloon)

        self.task_bar = TaskBar(self)

    def set_icon(self, path):
        icon = wx.Icon(wx.Bitmap(path))
        self.SetIcon(icon, 'Skybility Ucnp')
        self.ShowBalloon('Ucnp', 'Skybility', msec=2)

    def CreatePopupMenu(self):
        menu = wx.Menu()
        # self.task_bar.create_menu_item(menu, 'Show Gui', self.show_gui)

        # start service is equal to sign in in function
        self.task_bar.create_menu_item(menu, 'Start Service', self._app.login)

        # stop service is equal to sign out in function
        self.task_bar.create_menu_item(menu, 'Stop Service', self._app.logout)
        self.task_bar.chk_item_click(menu, 'Client List',
                                     self.block_special_user,
                                     self._app.connection)
        self.task_bar.chk_item_click(menu, 'Recent Content',
                                     self.show_recent_content,
                                     self._app.pipe_data())
        self.task_bar.create_menu_item(menu, 'Sign Out',
                                       self.sign_out_and_show_gui)
        self.task_bar.create_menu_item(menu, 'Log', self.show_log)
        self.task_bar.create_menu_item(menu, 'Exit', self.on_exit)

        return menu

    def sign_out_and_show_gui(self, event):
        self.show_gui(event)
        self._app.logout(event)

    @property
    def app(self):
        return self._app

    def click_if_not_clicked(self, event):
        pass

    def show_recent_content(self, index):
        def recent_content(event):
            content = self._app.pipe_data()[index]
            ui = NormalUi(self._frame, -1, "LOG", size=(659, 500))
            text = wx.TextCtrl(ui, -1, style=wx.TE_MULTILINE)
            text.write(content)
            ui.Show()

        return recent_content

    def show_log(self, event):
        content = ''
        file_path = os.path.join(user_home, 'ucnp_client', 'log',
                                 'up_utils.log')
        if os.path.isfile(file_path):
            with open(file_path, 'r') as f:
                content = f.read()

        ui = NormalUi(self._frame, -1, "LOG", size=(659, 500))
        text = wx.TextCtrl(ui, -1, style=wx.TE_MULTILINE)
        logging.error(
            'show log for pipe data id is {0}'.format(id(self._app.pipe_data)))
        text.write(content)
        ui.Show()

    def block_special_user(self, index):
        def block_user_list(event):
            if index in self._app.connection.ignore_clients:
                self._app.connection.remove_ignore_client(index)
            elif index not in self._app.connection.ignore_clients:
                self._app.connection.add_ignore_client(index)

        return block_user_list

    def on_exit(self, event):
        self.Destroy()
        self.RemoveIcon()
        self._app.stop()
        self._app.Destroy()
        self._frame.Close(True)
        self._app.Close(True)
        logging.info('Stop ucnp Skybility now')
        wx.Exit()

    def show_balloon(self, event):
        # self.CreatePopupMenu()
        self.ShowBalloon('Ucnp', 'Skybility', msec=2)

    def show_gui(self, event):
        self._app.Show(True)
        self._app.Iconize(iconize=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File: loop.py.py
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""

from time import sleep


class Loop:
    def __init__(self, func, interval, *args):
        self.func = func
        self.interval = int(interval)
        self._exit = False
        self._execute_once = False
        self.args = args

    def start(self):
        if self._execute_once:
            ret = self.func(self.args)
            yield ret
            return

        while not self._exit:
            if self.args:
                ret = self.func(self.args)
            else:
                ret = self.func()

            yield ret

            if self.interval:
                sleep(self.interval)

    def stop(self):
        self._exit = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File: logging.py.py
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""
import os
import logging


def __init_log(log_level=logging.INFO, logger_name=None,
               log_dir=os.path.join("log", "client")):
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)
    logger.propagate = False

    formatter = logging.Formatter(
        '%(asctime)s [%(name)s]: <%(levelname)s> %(message)s '
        '[%(filename)s:%(lineno)d:%(funcName)s]')

    log_dir = log_dir
    if not os.path.exists(log_dir):
        os.makedirs(log_dir, exist_ok=True)

    log_file = logger_name
    if not log_file:
        log_file = 'up_utils'

    log_file = os.path.join(log_dir, log_file)

    if not log_file.endswith('.log'):
        log_file += '.log'

    handler = logging.FileHandler(log_file)
    handler.setFormatter(formatter)

    handler.setLevel(log_level)
    handler.name = 'test'

    logger.addHandler(handler)


def __set_proc_name(proc_name):
    logging.getLogger(proc_name)


setattr(logging, 'init_log', __init_log)
setattr(logging, 'set_proc_name', __set_proc_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File: uri.py.py
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""
import json
import requests
import socketio
import pyperclip
from client.uri.loop import Loop
from client.uri.os_property import os_and_user_name
from client.client_log.logging import logging


def get_chipboard():
    ret = pyperclip.waitForNewPaste()
    return ret


def log_detail(*args):
    # logging.error(f'callback recorded {args}')
    pass


USER_LIST = []


class AppRequest:
    def __init__(self, url):
        if not isinstance(url, str):
            url = str(url)

        self.base_url = url
        self.token = ''

    def register(self, user, pw, pw_ag):
        register_url = self.base_url + '/register'
        data = {'username': user, 'password': pw, 'password2': pw_ag}
        headers = {
            'content-type': 'application/json',
            }
        response = requests.post(register_url, data=json.dumps(data),
                                 headers=headers)
        return response.text

    def login(self, user, pw):
        login_url = self.base_url + '/login'
        data = {'username': user, 'password': pw}
        headers = {
            "content-type": "application/json",
            }
        try:
            response = requests.post(login_url, data=json.dumps(data),
                                     headers=headers)
            self.token = response.json()
        except Exception as e:
            logging.error(f'Failed to login to server, details is {e}')
            return {'code': 404,
                    'message': f'Failed to login to server, details is {e}'}

        return response.json()


class SocketClient:
    def __init__(self, url, pipe_data, connection):
        if not isinstance(url, str):
            url = str(url)
        self.url = url
        self._pipe_data = pipe_data
        self._connection = connection
        self.sio = socketio.Client()
        self._loop = None
        self.os_name = os_and_user_name()

    def connect(self, token):
        self.sio.connect(
            self.url, headers={'Authorization': token, 'client': self.os_name},
            namespaces=['/ucnp'])

    def start(self):
        self._loop = Loop(get_chipboard, interval=0)
        logging.debug('start waiting for ctrl + c')
        for ret in self._loop.start():
            data = {"ignore_sid": self._connection.ignored_sids(),
                    "content": ret}
            self.sio.emit('message', data, callback=log_detail,
                          namespace='/ucnp')

        self.sio.wait()

    def stop(self):
        logging.info('stop monitor and socket client')
        if self._loop:
            self._loop.stop()

        self.sio.disconnect()

    def register_namespace(self):
        self.sio.register_namespace(
            UriGet(self._pipe_data, self._connection, '/ucnp'))
        logging.info('Monitor namespace [ucnp] for user successfully started '
                     '========')
        self.sio.wait()


class UriGet(socketio.ClientNamespace):
    def __init__(self, pipe_data, connection, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._pipe_data = pipe_data
        self._connection = connection

    def on_connect(self):
        pass

    def on_disconnect(self):
        pass

    def on_message(self, data):
        self._pipe_data.add(data)
        pyperclip.copy(data)

    def on_online_users(self, data):
        self._connection.update(data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File:
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""

import time


class Connection:
    INSTANCE = None

    def __new__(cls, *args, **kwargs):
        if not cls.INSTANCE:
            cls.INSTANCE = super().__new__(cls)

        return cls.INSTANCE

    def __init__(self):
        # ['all_clients']
        self._current_clients = []

        # ['hostname', 'hostname2]
        self._ignore_clients = []

        #  {'hostname': ['sid1', 'sid2]}
        self._client_and_sid = {}

    def ignored_sids(self):
        ignored_sids = []
        for client in self.ignore_clients:
            ignored_sids.extend(self._client_and_sid.get(client, []))

        return ignored_sids

    def update(self, data):
        self._client_and_sid = {}
        self._current_clients = []
        for client in data:
            for hostname, sid in client.items():
                if hostname not in self._client_and_sid:
                    self._client_and_sid[hostname] = [sid]
                else:
                    self._client_and_sid[hostname].append(sid)

                if hostname not in self._current_clients:
                    self._current_clients.append(hostname)

    @property
    def current_clients(self):
        return self._current_clients

    @property
    def ignore_clients(self):
        return self._ignore_clients

    def add_ignore_client(self, ignore_client):
        if ignore_client not in self._current_clients:
            return

        if isinstance(ignore_client, list):
            self._ignore_clients.extend(ignore_client)

        elif isinstance(ignore_client, str):
            self._ignore_clients.append(ignore_client)

    def remove_ignore_client(self, ignore_client):
        if ignore_client not in self._current_clients:
            return

        if isinstance(ignore_client, list):
            for x in ignore_client:
                self._ignore_clients.remove(ignore_client)

        elif isinstance(ignore_client, str):
            self._ignore_clients.remove(ignore_client)


class PipeData:
    def __init__(self):
        self._data = ['', '', '', '', '']

    def add(self, data):
        if not len(data):
            return

        now = time.strftime('%Y-%m-%d %H:%M:%S : ')
        data = now + data
        self._data.insert(0, data)

    def __call__(self, *args, **kwargs):
        ret = []
        data = self._data[:5]
        for _d in data:
            if len(_d):
                ret.append(_d)
        return ret


connection = Connection()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
File: __init__.py.py
Author: Scott Yang(Scott)
Email: yangyingfa@skybility.com
Copyright: Copyright (c) 2020, Skybility Software Co.,Ltd. All rights reserved.
Description:
"""



import setuptools

long_description = ""
with open("README.md", 'r') as f:
    long_description = f.read()

setuptools.setup(
    name="ucnp-skybility",
    version="2.0.2",
    author="skybility ha team",
    url="https://gitlab.skybilityha.com/experiments/ucnp",
    author_email="support@skybilityha.com",
    description="this is a client to ucnp server for linux platform only",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        "pyperclip>=1.8.0",
        "python-engineio>=3.13.2",
        "python-socketio>=4.6.0",
        "requests>=2.24.0",
        "wxPython>=4.0.3",
        "pypubsub>=4.0.3",
        "pinyin>=0.4.0",
        ],
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        ],
    python_requires=">=3.5",
    scripts=['client/ucnp_start']
    )



import wx


########################################################################
class ClipboardPanel(wx.Panel):
    """"""

    # ----------------------------------------------------------------------
    def __init__(self, parent):
        """Constructor"""
        wx.Panel.__init__(self, parent)

        lbl = wx.StaticText(self, label="Enter text to copy to clipboard:")
        self.text = wx.TextCtrl(self, style=wx.TE_MULTILINE)
        copyBtn = wx.Button(self, label="Copy")
        copyBtn.Bind(wx.EVT_BUTTON, self.onCopy)
        copyFlushBtn = wx.Button(self, label="Copy and Flush")
        copyFlushBtn.Bind(wx.EVT_BUTTON, self.onCopyAndFlush)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(lbl, 0, wx.ALL, 5)
        sizer.Add(self.text, 1, wx.EXPAND)
        sizer.Add(copyBtn, 0, wx.ALL | wx.CENTER, 5)
        sizer.Add(copyFlushBtn, 0, wx.ALL | wx.CENTER, 5)
        self.SetSizer(sizer)

    # ----------------------------------------------------------------------
    def onCopy(self, event):
        """"""
        self.dataObj = wx.TextDataObject()
        self.dataObj.SetText(self.text.GetValue())
        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(self.dataObj)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox("Unable to open the clipboard", "Error")

    # ----------------------------------------------------------------------
    def onCopyAndFlush(self, event):
        """"""
        self.dataObj = wx.TextDataObject()
        self.dataObj.SetText(self.text.GetValue())
        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(self.dataObj)
            wx.TheClipboard.Flush()
        else:
            wx.MessageBox("Unable to open the clipboard", "Error")

        self.GetParent().Close()


########################################################################
class ClipboardFrame(wx.Frame):
    """"""

    # ----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        wx.Frame.__init__(self, None, title="Clipboard Tutorial")
        ClipboardPanel(self)
        self.Show()


if __name__ == "__main__":
    app = wx.App(False)
    frame = ClipboardFrame()
    app.MainLoop()
